c Main program for solving Shell Dual Problem 
c by use of the exact penalty function method.
c The penalty coefficient is chosen a priori 
c by the user.
c This is an example on how to use entries 
c avoiding the use of a common block.
        double precision, dimension(:), allocatable :: x
        double precision options(13), f
        integer n
        logical flg,flfc,flgc
        external dualshel,dualobjf,dualobjg,null
          flg=.true.
c Flags for constraints are set to 0:
          flfc=.false.
          flgc=.false.
c Allocate array X:          
         n=15
         allocate (x(n))
c Initialize the problem constants:
         call dualshel(x)
c Use the default optional parameters:
         call soptions(options)
c Call the solver:         
         call solvopt(n,x,f,dualobjf,flg,dualobjg,options,
     1    flfc,null,flgc,null)
c Display the results:
      print '(//5x,a)','Function Value ===== Evaluations == Iterations'
      print '(/ 1x,g22.15,2x,i5,2h +,i5,5x,i5)',
     1   f,idnint(options(10)),idnint(options(11)),idnint(options(9))
      print '(//1x,16hOptimum Point X:,//((1x,g22.15)))', x

        deallocate (x)
        end

c  Initialization routine:
      subroutine dualshel(x)
      double precision f,f0,s,penalty
      double precision x(*)
      double precision g(*)
      double precision a(10,5), b(10), c(5,5), d(5), e(5)
      integer im, i, j
      save a,b,c,d,e,im, penalty
c A priori given penalty coefficient:
      data penalty/1.d3/
      data a /
     c  -16.d0,   0.d0,-3.5d0,   0.d0,   0.d0,   
     c    2.d0,  -1.d0, -1.d0,   1.d0,   1.d0,
     c    2.d0,  -2.d0,  0.d0,  -2.d0,  -9.d0,
     c    0.d0,  -1.d0, -2.d0,   2.d0,   1.d0,
     c    0.d0,   0.d0,  2.d0,   0.d0,  -2.d0,
     c   -4.d0,  -1.d0, -3.d0,   3.d0,   1.d0,
     c    1.d0,   .4d0,  0.d0,  -4.d0,   1.d0,
     c    0.d0,  -1.d0, -2.d0,   4.d0,   1.d0,
     c    0.d0,   2.d0,  0.d0,  -1.d0,  -2.8d0,
     c    0.d0,  -1.d0, -1.d0,   5.d0,   1.d0/
      data   b / -40.d0, -2.d0, -.25d0, -4.d0, -4.d0, -1.d0, -40.d0, 
     c        -60.d0, 5.d0, 1.d0 / 
      data   c /  30.d0, -20.d0, -10.d0,  32.d0, -10.d0,
     c        -20.d0,  39.d0,  -6.d0, -31.d0,  32.d0,
     c        -10.d0,  -6.d0,  10.d0,  -6.d0, -10.d0,
     c         32.d0, -31.d0,  -6.d0,  39.d0, -20.d0,
     c        -10.d0,  32.d0, -10.d0, -20.d0,  30.d0 / 
      data d /  4.d0, 8.d0, 10.d0, 6.d0, 2.d0 / 
      data e /  -15.d0, -27.d0, -36.d0, -18.d0, -12.d0 / 
c Standard starting point:
      do i=1,15
       x(i)=1.d-4
      enddo
      x(12)=6.d1 
      return
      
c Penalty function:
      entry dualobjf(x,f)

c Objective function:
      f0=0.d0
      do i=1,5
         do j=1,5
            f0=f0+c(i,j)*x(i)*x(j)
         enddo
         f0=f0+d(i)*x(i)**3*2.d0
      enddo
      do i=6,15
         f0=f0-b(i-5)*x(i)
      enddo
c Penalty item:
      f=0.d0
      im=0
      do i=1,20
        if(i.le.5) then
          s=0.d0
          do  j=1,10
             s=s+a(j,i)*x(j+5)
          enddo
          do  j=1,5
             s=s-2.d0*c(j,i)*x(j)
          enddo
          s=s-e(i)-3.d0*d(i)*x(i)**2
        else
          s=-x(i-5)
        endif
        if (s.gt.f) then
c The maximal residual:
          f=s
c The constraint with the maximal residual:
          im=i
        endif    
      enddo    
c The sum of the above two items:
      f=f0+penalty*f
      return
      
c Gradient of the penalty function:
      entry dualobjg(x,g)

      do i=1,15
         if(i.le.5) then
           g(i)=6.d0*d(i)*x(i)**2
           s=0.d0
           do j=1,5
             s=s+c(i,j)*x(j)
           enddo
           g(i)=g(i)+2.d0*s
         else
           g(i)=-b(i-5)
         endif  
      enddo
      if (im.gt.0) then
        if(im.le.5) then
          do j=1,15
            if(j .le. 5)  then
                g(j)=g(j)-2.d0*c(j,im)*penalty
            else    
                g(j)=g(j)+a(j-5,im)*penalty
            endif    
          enddo
          g(im)=g(im)-6.d0*d(im)*x(im)*penalty
        else
          g(im-5)=g(im-5)-penalty
        endif
      endif
      return
      end
